#!/usr/bin/env python
# -*- mode: python; coding: utf-8 -*-


import sys
import os
from distutils.core import setup, Extension
from distutils.command.install_data import install_data

from odinbookreader.config import program_version

BUILD_FB2_SUPPORT = 1

class my_install_data(install_data):
    # for install data files to library dir
    def run(self):
        #need to change self.install_dir to the actual library dir
        install_cmd = self.get_finalized_command('install')
        self.install_dir = getattr(install_cmd, 'install_lib')
        return install_data.run(self)

ext_modules = []

## build FictionBook2 support
if BUILD_FB2_SUPPORT:
    xml2_libs = os.popen('xml2-config --libs', 'r').read().split()
    xml2_cflags = os.popen('xml2-config --cflags', 'r').read().split()
    ext = Extension('odinbookreader.fb2wrap',
                    ['odinbookreader/fb2parser.c',
                     'odinbookreader/fb2parser_wrap.c'],
                    extra_compile_args = xml2_cflags,
                    extra_link_args = xml2_libs,
                    )
    ext_modules.append(ext)

setup(
    name = 'odin',
    version = program_version,
    url = 'http://odin.sourceforge.net',
    author = 'matimatik',
    author_email = 'matimatik@lavabit.com',
    description = 'Odin is a PyGTK-based GUI book reader.',
    license = 'GPL',
    scripts = ['odin',],
    packages = ['odinbookreader', ],
    ext_modules = ext_modules,
    cmdclass = {'install_data': my_install_data},
    data_files = [('odinbookreader',
                   ['odinbookreader/odinbookreader.glade']), ]
    )

